'use strict';

exports.__esModule = true;
exports.objDeepMerge = exports.queryTypeOf = exports.$log = exports.xeUtils = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.debounce = debounce;
exports.asyncHandlerLockWrap = asyncHandlerLockWrap;
exports.getDeepValue = getDeepValue;
exports.formatDateString = formatDateString;
exports.formatNumber = formatNumber;
exports.formatRate = formatRate;

var _xeUtils = require('xe-utils');

var _xeUtils2 = _interopRequireDefault(_xeUtils);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

/* // Decimal, UnitTranslate, weightUnits, weightUnitCodes...
export * from './unit-translate.js' // 避免传递链过长 建议单独引用 */

exports.xeUtils = _xeUtils2.default;

/**
 * 针对本地开发 console.log 添加颜色打印
 */
/*
export const $log = (function() {
  const isDEV = process.env.NODE_ENV !== 'production' // 开发环境
  if (!isDEV) {
    // eslint-disable-next-line
    // @ts-ignore
    return (content = 'content', title = 'title', background = '#007bfc') => {}
  }
  return (content = 'content', title = 'title', background = '#007bfc') => {
    content = typeof content === 'object' ? JSON.stringify(content) : content
    console.log.apply(null, [`%c ${title}： %c ${content} `, 'padding: 1px; border-radius: 10px; color: #fff; background: #9159B2;', `padding: 1px; border-radius: 10px; color: #fff; background: ${background};`])
  }
})()*/
var $log = exports.$log = function $log() {
  var content = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 'content';
  var title = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'title';
  var background = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : '#007bfc';

  content = (typeof content === 'undefined' ? 'undefined' : _typeof(content)) === 'object' ? JSON.stringify(content) : content;
  console.log.apply(null, ['%c ' + title + '\uFF1A %c ' + content + ' ', 'padding: 1px; border-radius: 10px; color: #fff; background: #9159B2;', 'padding: 1px; border-radius: 10px; color: #fff; background: ' + background + ';']);
};

/**
 * @desc 函数防抖
 * @param func 函数
 * @param wait 延迟毫秒数
 * @param immediate true/false (是/否)即执行
 */
function debounce(func, wait, immediate) {
  var timeout = void 0;

  return function () {
    /* eslint-disable */
    // @ts-ignore
    var context = this;
    var args = arguments;
    /* eslint-enable */
    if (timeout) clearTimeout(timeout);
    if (immediate) {
      var callNow = !timeout;
      timeout = setTimeout(function () {
        timeout = null;
      }, wait);
      if (callNow) func.apply(context, args);
    } else {
      timeout = setTimeout(function () {
        func.apply(context, args);
      }, wait);
    }
  };
}
/**
 * 请求确切的类型
 */
var queryTypeOf = exports.queryTypeOf = function queryTypeOf(obj) {
  return Object.prototype.toString.call(obj).slice(8, -1);
};
/**
 * 深层数据的集合进行有机合并
 * @param objs[] 需要合并的 对象集合 （后面的覆盖前面的参数）
 * @returns {{}}
 */
var objDeepMerge = exports.objDeepMerge = function objDeepMerge() {
  for (var _len = arguments.length, objs = Array(_len), _key = 0; _key < _len; _key++) {
    objs[_key] = arguments[_key];
  }

  var res = {};
  var combine = function combine(opt) {
    for (var prop in opt) {
      // eslint-disable-next-line
      if (opt.hasOwnProperty(prop)) {
        // if (Object.prototype.toString.call(opt[prop]) === '[object Object]') {
        if (queryTypeOf(opt[prop]) === 'Object') {
          res[prop] = objDeepMerge(res[prop], opt[prop]);
        } else {
          res[prop] = opt[prop];
        }
      }
    }
  };
  objs.map(function (opts) {
    combine(opts);
  });
  return res;
};

/**
 * 异步操作避免重复触发加锁
 * @param syncFn
 * @returns {(function(...[*]): Promise<void>)|*}
 */
function asyncHandlerLockWrap(syncFn) {
  var is_lock = false;
  /* eslint-disable */
  // @ts-ignore
  return _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var context,
        _len2,
        args,
        _key2,
        _args = arguments;

    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // @ts-ignore
            context = this;

            if (!is_lock) {
              is_lock = true;
              // @ts-ignore

              for (_len2 = _args.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                args[_key2] = _args[_key2];
              }

              Promise.resolve().then(syncFn.bind.apply(syncFn, [context].concat(args))).finally(function () {
                is_lock = false;
              });
            }

          case 2:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));
  /* eslint-enable */
}

/**
 * 提取深层数据的值 (防止中间项 不存在导致的报错)
 * @param obj 【检测对象】
 * @param keyArr 【需要获取该对象内部数据key数组】
 * @returns {*}
 */
function getDeepValue(obj, keyArr) {
  return keyArr.reduce(function (acc, key) {
    return acc && acc[key];
  }, obj);
}

/**
 * 转化成需要展示的时间格式
 * @param date: string | Date | number
 * @param format: string 格式化 默认：MM/dd/yyyy
 *        format 输出日期格式(年份(yy|yyyy)、月份(M|MM自动补0)、天(d|dd自动补0)、12小时制(h|hh自动补0)、24小时制(H|HH自动补0)、分钟(m|mm自动补0)、秒(s|ss自动补0)、毫秒(S|SSS自动补0)、D当年的第几天、a/A上午下午、e/E星期几、w当年的第几周、W当月的第几周、q当年第几个季度、Z时区)
 * @returns {string}
 */
// export function formatDateString (date, format = 'MM/dd/yyyy HH:mm:ss') {
function formatDateString(date) {
  var format = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'MM/dd/yyyy';

  if (!date) return '-';
  return _xeUtils2.default.toDateString(date, format);
}

/**
 * 转化成带逗号的number格式
 * @param num
 * @param minimumFractionDigits 保留小数位
 * @returns {string}
 */
function formatNumber(num) {
  var minimumFractionDigits = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

  // return xeUtils.commafy(+num, { digits: minimumFractionDigits }) || '-'
  return (+num || 0).toLocaleString('zh', { minimumFractionDigits: minimumFractionDigits });
}

/**
 * 百分比 保留两位小数  eg: 99.567% -> 99.57%
 * @param rate 百分比数字/字符串
 * @param hasPercent 是否添加%
 * @returns {string}
 */
function formatRate(rate) {
  var hasPercent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : true;

  rate = rate ? rate.toString().replace('%', '') : '0';
  return parseFloat(rate).toFixed(2) + (hasPercent ? '%' : '');
}