'use strict';

exports.__esModule = true;
exports.sizeUnits = exports.weightUnitCodes = exports.weightUnits = exports.WeightCode = exports.DimensionsCode = exports.UnitTranslate = exports.Decimal = undefined;

var _decimal = require('decimal.js');

exports.Decimal = _decimal.Decimal;

// function conversionOfOz(oz: Decimal, unit = 16) {
//   return oz.div(unit).toNumber();
// }

// add加法 sub减法  mul乘法 div除法

var UnitTranslate = exports.UnitTranslate = {
  conversionOfNumber: function conversionOfNumber(arr /* : number[] */) {
    var voe = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 16;
    var num1 = arr[0],
        num2 = arr[1];

    if (num1 % 1 !== 0) {
      // num1 = num1向下取整； 借位 = 余数 * voe; num2 += 借位；
      var borrowArr = ('' + num1).split('.');
      num1 = +borrowArr[0];
      num2 = new _decimal.Decimal(+('0.' + borrowArr[1])).mul(voe).add(num2).toNumber();
    }

    if (num2 >= voe) {
      var num11 = new _decimal.Decimal(num1);
      // const _num2 = new Decimal(num2);
      var num1OfPlus = Math.floor(num2 / voe);
      var num2OfRem = num2 % voe;
      return [num11.plus(num1OfPlus).toNumber(), num2OfRem];
    }
    return [num1, num2];
  },

  // [16,8] 转换为 g/oz
  computedOzOrGForWeightArr: function computedOzOrGForWeightArr(arr /* : number[] */) {
    var voe = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 16;
    var num1 = arr[0],
        num2 = arr[1];

    var coe = new _decimal.Decimal(num1).mul(voe); // 16 * 16 = 256
    return coe.add(num2).toNumber(); // 256 + 8 = 264
  },

  // [16,8] 转换为 kg/lb
  computedLBSOrKgForWeightArr: function computedLBSOrKgForWeightArr(arr /* : number[] */) {
    var voe = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 16;

    var coe = new _decimal.Decimal(1).div(voe); // 1/16
    var num1 = arr[0],
        num2 = arr[1];

    return coe.mul(num2).add(num1).toNumber(); // 1/16 * 8 + 16 = 16.5
  },

  //  16.32
  computedWeightArrForLBSOrKg: function computedWeightArrForLBSOrKg(num /* : number */) /* : number[] */{
    var coe = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 16;

    var strNum = '' + num;
    var arrNum /* : (number | string)[] */ = strNum.split('.'); // [16,32]
    var numOne = arrNum[0]; // 16
    var numTwo = arrNum[1] || 0; // 32
    if (numTwo) {
      numTwo = new _decimal.Decimal(+('0.' + numTwo)).mul(coe).toNumber().toFixed(3); // 0.32*16 = 5.12
    } else {
      numTwo = 0;
    }
    return [+numOne, +numTwo]; // [16,5.12]
  },

  // 16.32
  computedWeightArrForGOrOz: function computedWeightArrForGOrOz(num /* : number */) /* : number[] */{
    var coe = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 16;

    var strNum = '' + num;
    var arrNum /* : (number | string)[] */ = strNum.split('.'); // [16,32]
    var decimalNum = arrNum[1] ? +('0.' + arrNum[1]) : 0; // 0.32
    var weightNum = +arrNum[0]; // 16
    var numOne = 0,
        numTwo = 0;

    numOne = Math.floor(weightNum / coe); // 1
    numTwo = new _decimal.Decimal(weightNum % coe).add(decimalNum).toNumber(); // 0.32

    return [numOne, numTwo]; // [1,0.32]
  },
  translateWeightArrOfWeight: function translateWeightArrOfWeight(num /* : number */, unit /* : string */) /* : number[] */{
    var returnArr /* : number[] */ = [0, 0];
    switch (unit) {
      case 'LBS':
      case 'lbs':
      case 'lb':
        returnArr = this.computedWeightArrForLBSOrKg(num, 16);
        break;
      case 'OZS':
      case 'ozs':
      case 'oz':
        returnArr = this.computedWeightArrForGOrOz(num, 16);
        break;
      case 'KGS':
      case 'kgs':
      case 'kg':
        returnArr = this.computedWeightArrForLBSOrKg(num, 1000);
        break;
      case 'GS':
      case 'gs':
      case 'g':
        returnArr = this.computedWeightArrForGOrOz(num, 1000);
        break;
      default:
        break;
    }
    return returnArr;
  }
};

// 尺寸
var DimensionsCode = exports.DimensionsCode = {
  CM: 'cm',
  In: 'in'

  // 重量
};var WeightCode = exports.WeightCode = {
  G: 'g',
  Oz: 'oz'

  // 重量单位切换
};var weightUnits = exports.weightUnits = [{
  label: 'lb/oz',
  value: 0,
  param: 'oz',
  code: ['lb', 'oz'],
  step: 16,
  length: 3
}, {
  label: 'kg/g',
  value: 1,
  param: 'g',
  code: ['kg', 'g'],
  step: 1000,
  length: 3
}];

// 重量单位名称数组
var weightUnitCodes = exports.weightUnitCodes = weightUnits.reduce(function (res, v) {
  res[v.param] = v.code;
  return res;
}, {});

// 尺寸单位切换
var sizeUnits = exports.sizeUnits = [
/*{
label: string
param: 'in' | 'cm'
value: number
code: 'in' | 'cm'
}*/
{
  label: 'in',
  value: 0,
  param: 'in',
  code: 'in'
}, {
  label: 'cm',
  value: 1,
  param: 'cm',
  code: 'cm'
}];